"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _PolkadotService_api;
Object.defineProperty(exports, "__esModule", { value: true });
exports.PolkadotService = void 0;
require("@moonbeam-network/api-augment");
const lodash_1 = require("lodash");
const polkadot_api_1 = require("./polkadot.api");
const polkadot_utils_1 = require("./polkadot.utils");
class PolkadotService {
    constructor(api) {
        _PolkadotService_api.set(this, void 0);
        __classPrivateFieldSet(this, _PolkadotService_api, api, "f");
    }
    static async create(ws) {
        return new PolkadotService(await (0, polkadot_api_1.getPolkadotApi)(ws));
    }
    static getChainMin(weight, unitsPerSecond) {
        return (0, polkadot_utils_1.calculateMin)(weight, unitsPerSecond);
    }
    getMetadata() {
        return {
            decimals: __classPrivateFieldGet(this, _PolkadotService_api, "f").registry.chainDecimals.at(0) || 12,
            symbol: __classPrivateFieldGet(this, _PolkadotService_api, "f").registry.chainTokens.at(0),
        };
    }
    getExistentialDeposit() {
        return __classPrivateFieldGet(this, _PolkadotService_api, "f").consts.balances?.existentialDeposit.toBigInt() || 0n;
    }
    async getAssetDecimals(assetId) {
        const meta = await this.getAssetMeta(assetId);
        return meta.decimals.toNumber();
    }
    async getAssetMeta(assetId) {
        // TODO: how to fix any?
        return __classPrivateFieldGet(this, _PolkadotService_api, "f").query.assets.metadata(assetId);
    }
    async subscribeToAccountInfo(account, callback) {
        return __classPrivateFieldGet(this, _PolkadotService_api, "f").query.system.account(account, callback);
    }
    async subscribeToBalance(account, callback) {
        return this.subscribeToAccountInfo(account, ({ data }) => callback(data.free.toBigInt() - data.miscFrozen.toBigInt()));
    }
    async getGenericBalance(account, { pallet, function: fn, getParams, path, calc }) {
        const response = await __classPrivateFieldGet(this, _PolkadotService_api, "f").query[pallet][fn](...getParams(account));
        if (response.isEmpty) {
            return 0n;
        }
        // TODO: improve types here and in balance interfaces
        const unwrapped = response.unwrap?.() || response;
        return calc(path.length ? (0, lodash_1.get)(unwrapped, path) : unwrapped);
    }
    async getAssetMinBalance({ pallet, function: fn, params, path, }) {
        const details = await __classPrivateFieldGet(this, _PolkadotService_api, "f").query[pallet][fn](...params);
        if (details.isEmpty) {
            return 0n;
        }
        return (0, lodash_1.get)(details, path).toBigInt();
    }
    getXcmExtrinsic(account, amount, { pallet, extrinsic, getParams }, fee, primaryAccount) {
        let transferExtrinsic = __classPrivateFieldGet(this, _PolkadotService_api, "f").tx[pallet][extrinsic](
        // TODO: check issue with types and if we can fix it
        // @ts-ignore
        ...getParams(account, amount, fee));
        if (primaryAccount) {
            transferExtrinsic = __classPrivateFieldGet(this, _PolkadotService_api, "f").tx.proxy.proxy(primaryAccount, null, transferExtrinsic);
        }
        return transferExtrinsic;
    }
    async getAssetFee(id, weight) {
        const type = await this.getAssetType(id);
        if (!type) {
            return 0n;
        }
        const unitsPerSecond = await __classPrivateFieldGet(this, _PolkadotService_api, "f").query.assetManager.assetTypeUnitsPerSecond(type);
        return (0, polkadot_utils_1.calculateMin)(weight, unitsPerSecond.unwrapOrDefault().toBigInt());
    }
    async getAssetType(id) {
        const type = await __classPrivateFieldGet(this, _PolkadotService_api, "f").query.assetManager.assetIdType(id);
        return type.unwrapOr(undefined);
    }
    async subscribeToAssetsBalanceInfo(account, configGetter, callback) {
        const assetsArray = Object.values(configGetter.assets);
        const ids = assetsArray.map((asset) => asset.id);
        const metadata = (await __classPrivateFieldGet(this, _PolkadotService_api, "f").query.assets.metadata.multi(ids));
        return this.subscribeToAccountBalances(account, ids, (data) => {
            callback(data.map((balance, index) => {
                const asset = assetsArray[index];
                const meta = metadata[index];
                const { chains: [chain], from, } = configGetter.deposit(asset.originSymbol);
                // TODO: remove as
                const { origin } = from(chain.chain);
                return {
                    asset,
                    balance: balance.balance.toBigInt(),
                    meta: {
                        decimals: asset.isNative
                            ? origin.decimals
                            : meta.decimals.toNumber(),
                        symbol: asset.isNative
                            ? asset.originSymbol
                            : meta.symbol.toHuman()?.toString() || '',
                        originSymbol: asset.isNative
                            ? asset.originSymbol
                            : meta.name.toHuman()?.toString(),
                    },
                    origin,
                };
            }));
        });
    }
    async subscribeToAccountBalances(account, ids, callback) {
        const keys = ids.map((x) => [x, account]);
        return __classPrivateFieldGet(this, _PolkadotService_api, "f").query.assets.account.multi(keys, (res) => {
            const response = res.map((balance) => balance.unwrapOrDefault());
            callback(response);
        });
    }
}
exports.PolkadotService = PolkadotService;
_PolkadotService_api = new WeakMap();
//# sourceMappingURL=PolkadotService.js.map