"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getCreateExtrinsic = exports.createTransactionEventHandler = exports.createExtrinsicEventHandler = exports.isXcmSdkWithdraw = exports.isXcmSdkDeposit = void 0;
const xcm_config_1 = require("@moonbeam-network/xcm-config");
const sdk_interfaces_1 = require("./sdk.interfaces");
function isXcmSdkDeposit(config) {
    return !!config.from;
}
exports.isXcmSdkDeposit = isXcmSdkDeposit;
function isXcmSdkWithdraw(config) {
    return !!config.to;
}
exports.isXcmSdkWithdraw = isXcmSdkWithdraw;
function createExtrinsicEventHandler(config, cb) {
    return ({ events = [], status, txHash }) => {
        const hash = txHash.toHex();
        if (status.isReady) {
            cb({ status: sdk_interfaces_1.ExtrinsicStatus.Sent, txHash: hash });
        }
        if (status.isInBlock) {
            const block = status.asInBlock.toString();
            events.forEach(({ event: { data, method, section } }) => {
                if (section === config.extrinsic.pallet &&
                    method === config.extrinsic.successEvent) {
                    if (method === xcm_config_1.PolkadotXcmExtrinsicSuccessEvent.Attempted) {
                        const eventData = data.at(0);
                        if (eventData.isIncomplete) {
                            cb({
                                status: sdk_interfaces_1.ExtrinsicStatus.Failed,
                                blockHash: block,
                                txHash: hash,
                                message: eventData.asIncomplete.toHuman().join('; '),
                            });
                            return;
                        }
                    }
                    cb({
                        status: sdk_interfaces_1.ExtrinsicStatus.Success,
                        blockHash: block,
                        txHash: hash,
                    });
                }
                if (section === 'system' && method === 'ExtrinsicFailed') {
                    cb({
                        status: sdk_interfaces_1.ExtrinsicStatus.Failed,
                        blockHash: block,
                        txHash: hash,
                        message: data.join('; '),
                    });
                }
            });
        }
    };
}
exports.createExtrinsicEventHandler = createExtrinsicEventHandler;
async function createTransactionEventHandler(ethersSigner, txHash, cb, skipSentEvent = false) {
    if (!ethersSigner.provider) {
        throw new Error('options.ethersSigner has not provider');
    }
    if (!skipSentEvent) {
        cb({ status: sdk_interfaces_1.ExtrinsicStatus.Sent, txHash });
    }
    const tx = await ethersSigner.provider.getTransactionReceipt(txHash);
    if (!tx) {
        setTimeout(() => createTransactionEventHandler(ethersSigner, txHash, cb, true), 2000);
        return;
    }
    if (tx.status === 1) {
        cb({
            status: sdk_interfaces_1.ExtrinsicStatus.Success,
            blockHash: tx.blockHash,
            txHash,
        });
    }
    else {
        cb({
            status: sdk_interfaces_1.ExtrinsicStatus.Failed,
            blockHash: tx.blockHash,
            txHash,
        });
    }
}
exports.createTransactionEventHandler = createTransactionEventHandler;
function getCreateExtrinsic({ account, config, foreignPolkadot, nativeAsset, polkadot, primaryAccount, }) {
    return async (amount) => {
        const fee = (0, xcm_config_1.isMultiCurrency)(config.extrinsic)
            ? await polkadot.getAssetFee(nativeAsset.id, config.origin.weight)
            : 0n;
        return foreignPolkadot.getXcmExtrinsic(account, amount, config.extrinsic, fee, primaryAccount);
    };
}
exports.getCreateExtrinsic = getCreateExtrinsic;
//# sourceMappingURL=sdk.utils.js.map