import '@moonbeam-network/api-augment';
import { Asset, BalanceConfig, Chain, ConfigGetter, ExtrinsicConfig, MinBalanceConfig } from '@moonbeam-network/xcm-config';
import { ApiPromise } from '@polkadot/api';
import { SubmittableExtrinsic, UnsubscribePromise } from '@polkadot/api/types';
import { AccountInfo } from '@polkadot/types/interfaces';
import { MoonbeamRuntimeXcmConfigAssetType, PalletAssetsAssetAccount, PalletAssetsAssetMetadata } from '@polkadot/types/lookup';
import { AssetBalanceInfo } from './polkadot.interfaces';
export declare class PolkadotService<Assets extends Asset = Asset, Chains extends Chain = Chain> {
    #private;
    constructor(api: ApiPromise);
    static create<Assets extends Asset = Asset, Chains extends Chain = Chain>(ws: string): Promise<PolkadotService<Assets, Chains>>;
    static getChainMin(weight: number, unitsPerSecond: bigint): bigint;
    getMetadata(): {
        decimals: number;
        symbol: Assets;
    };
    getExistentialDeposit(): bigint;
    getAssetDecimals(assetId: string): Promise<number>;
    getAssetMeta(assetId: string): Promise<PalletAssetsAssetMetadata>;
    subscribeToAccountInfo(account: string, callback: (info: AccountInfo) => void): UnsubscribePromise;
    subscribeToBalance(account: string, callback: (balance: bigint) => void): UnsubscribePromise;
    getGenericBalance(account: string, { pallet, function: fn, getParams, path, calc }: BalanceConfig<Assets>): Promise<bigint>;
    getAssetMinBalance({ pallet, function: fn, params, path, }: MinBalanceConfig): Promise<bigint>;
    getXcmExtrinsic(account: string, amount: bigint, { pallet, extrinsic, getParams }: ExtrinsicConfig<Assets>, fee?: bigint, primaryAccount?: string): SubmittableExtrinsic<'promise'>;
    getAssetFee(id: string, weight: number): Promise<bigint>;
    getAssetType(id: string): Promise<MoonbeamRuntimeXcmConfigAssetType | undefined>;
    subscribeToAssetsBalanceInfo(account: string, configGetter: ConfigGetter<Assets, Chains>, callback: (data: AssetBalanceInfo<Assets>[]) => void): UnsubscribePromise;
    subscribeToAccountBalances(account: string, ids: string[], callback: (balances: PalletAssetsAssetAccount[]) => void): UnsubscribePromise;
}
