import { isMultiCurrency, PolkadotXcmExtrinsicSuccessEvent, } from '@moonbeam-network/xcm-config';
import { ExtrinsicStatus, } from './sdk.interfaces';
export function isXcmSdkDeposit(config) {
    return !!config.from;
}
export function isXcmSdkWithdraw(config) {
    return !!config.to;
}
export function createExtrinsicEventHandler(config, cb) {
    return ({ events = [], status, txHash }) => {
        const hash = txHash.toHex();
        if (status.isReady) {
            cb({ status: ExtrinsicStatus.Sent, txHash: hash });
        }
        if (status.isInBlock) {
            const block = status.asInBlock.toString();
            events.forEach(({ event: { data, method, section } }) => {
                if (section === config.extrinsic.pallet &&
                    method === config.extrinsic.successEvent) {
                    if (method === PolkadotXcmExtrinsicSuccessEvent.Attempted) {
                        const eventData = data.at(0);
                        if (eventData.isIncomplete) {
                            cb({
                                status: ExtrinsicStatus.Failed,
                                blockHash: block,
                                txHash: hash,
                                message: eventData.asIncomplete.toHuman().join('; '),
                            });
                            return;
                        }
                    }
                    cb({
                        status: ExtrinsicStatus.Success,
                        blockHash: block,
                        txHash: hash,
                    });
                }
                if (section === 'system' && method === 'ExtrinsicFailed') {
                    cb({
                        status: ExtrinsicStatus.Failed,
                        blockHash: block,
                        txHash: hash,
                        message: data.join('; '),
                    });
                }
            });
        }
    };
}
export async function createTransactionEventHandler(ethersSigner, txHash, cb, skipSentEvent = false) {
    if (!ethersSigner.provider) {
        throw new Error('options.ethersSigner has not provider');
    }
    if (!skipSentEvent) {
        cb({ status: ExtrinsicStatus.Sent, txHash });
    }
    const tx = await ethersSigner.provider.getTransactionReceipt(txHash);
    if (!tx) {
        setTimeout(() => createTransactionEventHandler(ethersSigner, txHash, cb, true), 2000);
        return;
    }
    if (tx.status === 1) {
        cb({
            status: ExtrinsicStatus.Success,
            blockHash: tx.blockHash,
            txHash,
        });
    }
    else {
        cb({
            status: ExtrinsicStatus.Failed,
            blockHash: tx.blockHash,
            txHash,
        });
    }
}
export function getCreateExtrinsic({ account, config, foreignPolkadot, nativeAsset, polkadot, primaryAccount, }) {
    return async (amount) => {
        const fee = isMultiCurrency(config.extrinsic)
            ? await polkadot.getAssetFee(nativeAsset.id, config.origin.weight)
            : 0n;
        return foreignPolkadot.getXcmExtrinsic(account, amount, config.extrinsic, fee, primaryAccount);
    };
}
//# sourceMappingURL=sdk.utils.js.map